@@ -49,6 +49,9 @@ Par1Repairer::Par1Repairer(void)
 
 Par1Repairer::~Par1Repairer(void)
 {
+  delete [] (u8*)inputbuffer;
+  delete [] (u8*)outputbuffer;
+
   map<u32,DataBlock*>::iterator i = recoveryblocks.begin();
   while (i != recoveryblocks.end())
   {
@@ -82,6 +85,9 @@ Result Par1Repairer::Process(const CommandLine &commandline, bool dorepair)
   // How noisy should we be
   noiselevel = commandline.GetNoiseLevel();
 
+  // do we want to purge par files on success ?
+  bool purgefiles = commandline.GetPurgeFiles();
+
   // Get filesnames from the command line
   string par1filename = commandline.GetParFilename();
   const list<CommandLine::ExtraFile> &extrafiles = commandline.GetExtraFiles();
@@ -223,6 +229,12 @@ Result Par1Repairer::Process(const CommandLine &commandline, bool dorepair)
     }
   }
 
+  if (purgefiles == true)
+  {
+    RemoveBackupFiles();
+    RemoveParFiles();
+  }
+
   return eSuccess;
 }
 
@@ -253,6 +265,8 @@ bool Par1Repairer::LoadRecoveryFile(string filename)
     cout << "Loading \"" << name << "\"." << endl;
   }
 
+  parlist.push_back(filename);
+
   bool havevolume = false;
   u32 volumenumber = 0;
 
@@ -324,7 +338,7 @@ bool Par1Repairer::LoadRecoveryFile(string filename)
           ||
           (fileheader.datasize && (fileheader.dataoffset < sizeof(fileheader) || fileheader.dataoffset + fileheader.datasize > filesize))
           ||
-          (fileheader.datasize && (fileheader.filelistoffset <= fileheader.dataoffset && fileheader.dataoffset < fileheader.filelistoffset+fileheader.filelistsize || fileheader.dataoffset <= fileheader.filelistoffset && fileheader.filelistoffset < fileheader.dataoffset + fileheader.datasize)))
+          (fileheader.datasize && ((fileheader.filelistoffset <= fileheader.dataoffset && fileheader.dataoffset < fileheader.filelistoffset+fileheader.filelistsize) || (fileheader.dataoffset <= fileheader.filelistoffset && fileheader.filelistoffset < fileheader.dataoffset + fileheader.datasize))))
         break;
 
       // Check the size of the file list
@@ -503,7 +517,7 @@ bool Par1Repairer::LoadOtherRecoveryFiles(string filename)
 
   // Search for additional PAR files
   string wildcard = name + ".???";
-  list<string> *files = DiskFile::FindFiles(path, wildcard);
+  list<string> *files = DiskFile::FindFiles(path, wildcard, false);
 
   for (list<string>::const_iterator s=files->begin(); s!=files->end(); ++s)
   {
@@ -518,9 +532,9 @@ bool Par1Repairer::LoadOtherRecoveryFiles(string filename)
       // Check the the file extension is the correct form
       if ((tail[0] == 'P' || tail[0] == 'p') &&
           (
-            (tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r')
+            ((tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r'))
             ||
-            isdigit(tail[1]) && isdigit(tail[2])
+            (isdigit(tail[1]) && isdigit(tail[2]))
           ))
       {
         LoadRecoveryFile(filename);
@@ -549,9 +563,9 @@ bool Par1Repairer::LoadExtraRecoveryFiles(const list<CommandLine::ExtraFile> &ex
       // Check the the file extension is the correct form
       if ((tail[0] == 'P' || tail[0] == 'p') &&
           (
-            (tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r')
+            ((tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r'))
             ||
-            isdigit(tail[1]) && isdigit(tail[2])
+            (isdigit(tail[1]) && isdigit(tail[2]))
           ))
       {
         LoadRecoveryFile(filename);
@@ -652,9 +666,9 @@ bool Par1Repairer::VerifyExtraFiles(const list<CommandLine::ExtraFile> &extrafil
       // Check the the file extension is the correct form
       if ((tail[0] == 'P' || tail[0] == 'p') &&
           (
-            (tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r')
+            ((tail[1] == 'A' || tail[1] == 'a') && (tail[2] == 'R' || tail[2] == 'r'))
             ||
-            isdigit(tail[1]) && isdigit(tail[2])
+            (isdigit(tail[1]) && isdigit(tail[2]))
           ))
       {
         skip = true;
@@ -710,7 +724,13 @@ bool Par1Repairer::VerifyDataFile(DiskFile *diskfile, Par1RepairerSourceFile *so
   u64 filesize = diskfile->FileSize();
 
   if (filesize == 0)
+  {
+    if (noiselevel > CommandLine::nlSilent)
+    {
+      cout << "Target: \"" << name << "\" - empty." << endl;
+    }
     return true;
+  }
 
   // Search for the first file that is the correct size
   vector<Par1RepairerSourceFile*>::iterator sourceiterator = sourcefiles.begin();
@@ -1013,8 +1033,12 @@ bool Par1Repairer::RenameTargetFiles(void)
 
       // Rename it
       diskfilemap.Remove(targetfile);
+
       if (!targetfile->Rename())
         return false;
+
+      backuplist.push_back(targetfile);
+
       bool success = diskfilemap.Insert(targetfile);
       assert(success);
 
@@ -1386,3 +1410,67 @@ bool Par1Repairer::DeleteIncompleteTargetFiles(void)
 
   return true;
 }
+
+bool Par1Repairer::RemoveBackupFiles(void)
+{
+  vector<DiskFile*>::iterator bf = backuplist.begin();
+
+  if (noiselevel > CommandLine::nlSilent
+      && bf != backuplist.end())
+  {
+    cout << endl << "Purge backup files." << endl;
+  }
+
+  // Iterate through each file in the backuplist
+  while (bf != backuplist.end())
+  {
+    if (noiselevel > CommandLine::nlSilent)
+    {
+      string name;
+      string path;
+      DiskFile::SplitFilename((*bf)->FileName(), path, name);
+      cout << "Remove \"" << name << "\"." << endl;
+    }
+
+    if ((*bf)->IsOpen())
+      (*bf)->Close();
+    (*bf)->Delete();
+
+    ++bf;
+  }
+
+  return true;
+}
+
+bool Par1Repairer::RemoveParFiles(void)
+{
+  if (noiselevel > CommandLine::nlSilent
+      && parlist.size() > 0)
+  {
+      cout << endl << "Purge par files." << endl;
+  }
+
+  for (list<string>::const_iterator s=parlist.begin(); s!=parlist.end(); ++s)
+  {
+    DiskFile *diskfile = new DiskFile;
+
+    if (diskfile->Open(*s))
+    {
+      if (noiselevel > CommandLine::nlSilent)
+      {
+        string name;
+        string path;
+        DiskFile::SplitFilename((*s), path, name);
+        cout << "Remove \"" << name << "\"." << endl;
+      }
+
+      if (diskfile->IsOpen())
+        diskfile->Close();
+      diskfile->Delete();
+    }
+
+    delete diskfile;
+  }
+
+  return true;
+}
